// #name = TeamInfo Support Script
// #version = 1.12
// #date = November 27, 2001
// #status = release
// #description = Support script which keeps lists of all teams and players.  Also generates client messages of key events such as players connecting, dropping, joining teams, etc.  Useful for scripters.  See the ReadMe file for more info.
// #author = |TIA|Prometheus
// #warrior = Prometheus^
// #email = rrk@u.arizona.edu
// #web = http://www.tribe-tia.com
// #readme = scripts/autoexec/TeamInfo.txt
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  First our general utility functions:
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


function TeamName(%teamId) { return (%teamId == 0) ? "Observer" : detag($clTeamScore[%teamId,0]); }
function getPlayer(%clientId) { return $PlayerList[%clientId]; }
function ClientName(%clientId) { return rawName($PlayerList[%clientId].name); }
function ClientId(%name) { return $Name2Id[rawName(%name)]; }
function ClientTeamId(%clientId) { return $PlayerList[%clientId].teamId; }
function ClientTeamName(%clientId) { return TeamName($PlayerList[%clientId].teamId); }

function addFirstMessageCallback(%msgType, %func)
{
	for(%i = 0; (%afunc = $MSGCB[%msgType, %i]) !$= ""; %i++)
		if (%afunc $= %func) return;  // No duplicates
	for(%i=%i; %i != 0; %i--)  // shift all callbacks back one spot
		$MSGCB[%msgType, %i] = $MSGCB[%msgType, %i-1];
	$MSGCB[%msgType, 0] = %func;  // insert function at (now vacant) first position
}

function rawName(%clientName)  // strips off markup and tag info; assumes a valid T2 name/profile as input
{
	return stripChars(detag(%clientName),"\cp\co\c6\c7\c8\c9");
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Next our message handlers:
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function TIhandleJoin(%msgType, %msgString, %clientName, %clientId, %targetId, %isAI, %isAdmin, %isSuperAdmin, %isSmurf, %guid)
{
	%rawname = rawName(%clientName);
	$Name2Id[%rawname] = %clientId;
	$PlayerList[%clientId].oldteam = 0;

	if (($MyClientId == 0) && (getTaggedString(%msgString) $= "\c2Welcome to Tribes2 %1."))
	{
		$MyClientId = %clientId;
		$PlayerName = %rawname;
	}
}

function TIhandleDrop(%msgType, %msgString, %clientName, %clientId)
{
	%player = $PlayerList[%clientId];
	%oldteam = %player.oldteam;
	if (%oldteam == $MyTeamId) clientCmdServerMessage('MsgTeammateLeft',"", %clientId);
	else clientCmdServerMessage('MsgEnemyLeft', "", %clientId, %oldteam);

	%teamlistObj = %player.teamlistObj;
	if (isObject(%teamlistObj)) %teamlistObj.delete();
	clientCmdServerMessage('MsgTeamUpdate',"", %oldteam, nameToId("TeamList"@ %oldteam).getCount());
}

function TIhandleJoinTeam(%msgType, %msgString, %clientName, %teamName, %clientId, %teamId)
{
echo("Got Join Team "@ %clientId @" : "@ %teamId);
	%player = $PlayerList[%clientId];
	%oldteam = %player.oldteam;
	if (%teamId == -1) %teamId = 0;
	%player.oldteam = %teamId;

	%teamListName = "TeamList"@ %teamId;
	if (!isObject(%teamListName))
	{
		%newGroup = new SimGroup(%teamListName);
		TeamsGroup.add(%newGroup);
	}
	%player.teamlist = nameToId(%teamListName);

	%teamlistObj = %player.teamlistObj;
	if (%teamlistObj $= "")
	{
		%teamlistObj = new ScriptObject()
		{
			className = "TeamPlayer";
			name = detag(%clientName);  // Note that name is detagged
			guid = %player.guid;
			clientId = %clientId;
			player = %player;
	            isSmurf = %player.isSmurf;
		};
		%player.teamlistObj = %teamlistObj;
	}
	%teamListName.add(%teamListObj);

	if (%clientId == $MyClientId)  // that's me
	{
		$MyTeamId = %teamId;
		$MyTeamName = TeamName(%teamId);
		$EnemyTeamName = TeamName((%teamId+1) % $clTeamCount);  // rolls over team count
		clientCmdServerMessage('MsgIJoinedTeam', "", %teamId, %oldteam);
	}
	else
	{
		if (%teamId == $MyTeamId) clientCmdServerMessage('MsgTeammateJoined', "", %clientId);
		else clientCmdServerMessage('MsgEnemyJoined', "", %clientId, %teamId);

		if (%oldteam == $MyTeamId) clientCmdServerMessage('MsgTeammateLeft', "", %clientId);
		else clientCmdServerMessage('MsgEnemyLeft', "", %clientId, %oldteam);
	}

	clientCmdServerMessage('MsgTeamUpdate',"", %teamId, nameToId("TeamList"@ %teamId).getCount());
	clientCmdServerMessage('MsgTeamUpdate',"", %oldteam, nameToId("TeamList"@ %oldteam).getCount());
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Some internal support functions (Don't ever THINK of calling these -- they may disappear or change):
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function TeamsGroup::Reset()
{
	$MyClientId = 0;
	$MyTeamId = 0;
	$PlayerName = "";
	$MyTeamName = "";
	$EnemyTeamName = "";
	deletevariables("$Name2Id*");

	while (TeamsGroup.getCount() > 0) TeamsGroup.getObject(0).delete();
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//  Our actual package code:
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

package TeamInfo
{

function DispatchLaunchMode()
{
	addMessageCallback('MsgClientJoin',TIhandleJoin);
	addFirstMessageCallback('MsgClientDrop',TIhandleDrop);  // must be called before default handler
	addMessageCallback('MsgClientJoinTeam',TIhandleJoinTeam);

	parent::DispatchLaunchMode();
}

function IRCClient::onJoinGame(%address,%desc)  // always called just before a connect... I think
{
	clientCmdServerMessage('MsgConnecting',"",%address);
echo("Connecting to "@ %address @" ...");

	TeamsGroup::Reset();
	%newGroup = new SimGroup(TeamList0);
	TeamsGroup.add(%newGroup);

	parent::onJoinGame(%address,%desc);
}

function DisconnectedCleanup()
{
	TeamsGroup::Reset();
	parent::DisconnectedCleanup();
}

};

new SimGroup(TeamsGroup);
activatePackage(TeamInfo);
